"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const createModuleIdHOF_1 = require("./createModuleIdHOF");
// @ts-ignore
const ext_sdk_cliutils_1 = require("@hyext/ext-sdk-cliutils");
function filterModuleHOF(options) {
    if (options.mode === 'dev') {
        // 开发模式下打包所有
        return () => true;
    }
    const { projectRoot, filterModules, filterModulesVersion, imageBaseURL, SDKAlias = {} } = options;
    const modulesVersionHelper = new ext_sdk_cliutils_1.ModulesHelper({
        rootPath: projectRoot,
    });
    const moduleIdFactory = createModuleIdHOF_1.createModuleIdHOF({ projectRoot, SDKAlias })();
    return function processModuleFilter(module) {
        var _a, _b, _c;
        const modulePath = module['path'];
        // 这个文件baseBundle已经包含了, 如果多次打包, 运行的时候会报错
        if (modulePath.endsWith('/node_modules/metro/src/lib/polyfills/require.js')) {
            return false;
        }
        const moduleId = moduleIdFactory(modulePath);
        if (filterModules.has(moduleId)) {
            const version = modulesVersionHelper.getVersion(modulePath);
            if (filterModulesVersion.has(`${moduleId}@${version}`)) {
                throw new Error(`module ${modulePath} version dismatch: ${moduleId}@${version}`);
                process.exit(1);
            }
            return false;
        }
        /*
         * FIXME 更新0.62.2的hook
         */
        if (modulePath.indexOf('__prelude__') >= 0) {
            // const hook = Buffer.from(process.env.HYEXT_RN_HOOK || '', 'base64')
            // const hookText = hook.toString('ascii')
            // module.output[0]['data']['code'] = hookText
            return true;
        }
        let code = (_c = (_b = (_a = module === null || module === void 0 ? void 0 : module.output) === null || _a === void 0 ? void 0 : _a[0]) === null || _b === void 0 ? void 0 : _b['data']) === null || _c === void 0 ? void 0 : _c['code'];
        if (!code)
            return true;
        /*
         * FIXME 优化这段逻辑, 修复非压缩状态下替换失效的问题
         * 应该是放到 transform/babel 里比较合适
         */
        const nameMatch = code.match(/name[ ]*:[ ]*"(\S+?)"/);
        if (nameMatch && nameMatch.length >= 1) {
            const name = nameMatch[1];
            const typeMatch = code.match(/type[ ]*:[ ]*"(\S+?)"/);
            if (typeMatch && typeMatch.length >= 1) {
                const type = typeMatch[1];
                const relativeMatch = code.match(/httpServerLocation[ ]*:[ ]*"(\S+?)"/);
                if (relativeMatch && relativeMatch.length >= 1) {
                    const relativePath = relativeMatch[1];
                    //read image baseURI from process.env
                    const uri = imageBaseURL + relativePath + '/' + name + '.' + type;
                    code = code.replace(/\.exports=(\S+)}/, `.exports={uri:"${uri}"}}`);
                    module.output[0]['data']['code'] = code;
                }
            }
        }
        return true;
    };
}
exports.default = filterModuleHOF;
module.exports = filterModuleHOF;
