import path from 'path'

const pathSep = path.sep

export function defaultFactory() {
  const fileToIdMap = new Map();
  let nextId = 0;
  return (modulePath: string) => {
    let id = fileToIdMap.get(modulePath);
    if (typeof id !== 'number') {
      id = nextId++;
      fileToIdMap.set(modulePath, id);
    }
    return id;
  };
}

type Options = {
  projectRoot: string
  SDKAlias: { [aliasName: string]: string[] }
}


export function createModuleIdHOF({ projectRoot, SDKAlias = {} }: Options) {
  const genSDKReg = (SDKNames: string[]) => {
    const uniformNames = SDKNames.map(x => x.replace('/', '_'))
    return new RegExp(`node_modules_(${uniformNames.join('|')})`, 'ig')
  }

  return function createModuleIdFactory() {
    return (modulePath: string) => {
      //RN JS 全部以 RN_ 开头
      if (modulePath.includes('node_modules' + pathSep + 'react-native' + pathSep + 'Libraries' + pathSep)) {
        return `RN_${path.basename(modulePath, '.js')}`
      }

      let name = modulePath.replace(`${projectRoot}/`, '')

      // remove .js extendtion
      name = name.replace(/.js$/, '')

      // 路径分隔符换成 _
      const regExp = pathSep == '\\' ? new RegExp('\\\\', 'g') : new RegExp(pathSep, 'g')
      name = name.replace(regExp, '_')

      // 把业务/base jsbundle 里的非 node_modules 里的模块 id 都带上根目录名
      if (modulePath.includes(projectRoot) && !modulePath.includes('node_modules')) {
        name = path.basename(projectRoot) + '_' + name;
      }

      Object.entries(SDKAlias).forEach(([alias, SDKNames]) => {
        if (SDKNames.length <= 0) return

        const SDKReg = genSDKReg(SDKNames)
        if (SDKReg.test(name)) {
          name = name.replace(SDKReg, alias)
        }
      })

      return name;
    };
  }
}
