import { createModuleIdHOF } from './createModuleIdHOF'

// @ts-ignore
import { ModulesHelper } from '@hyext/ext-sdk-cliutils'

interface BaseOptions {
  projectRoot: string
}

interface DevOptions extends BaseOptions {
  filterModules?: Set<string>
  filterModulesVersion?: Set<string>
  imageBaseURL?: string
  SDKAlias?: { [aliasName: string]: string[] }
  mode: 'dev',
}

interface ProdOptions extends BaseOptions {
  filterModules: Set<string>
  filterModulesVersion: Set<string>
  imageBaseURL: string
  SDKAlias: { [aliasName: string]: string[] }
  mode: 'production'
}

export default function filterModuleHOF(options: DevOptions | ProdOptions) {
  if (options.mode === 'dev') {
    // 开发模式下打包所有
    return () => true
  }

  const { projectRoot, filterModules, filterModulesVersion, imageBaseURL, SDKAlias = {} } = options

  const modulesVersionHelper = new ModulesHelper({
    rootPath: projectRoot,
  })

  const moduleIdFactory = createModuleIdHOF({ projectRoot, SDKAlias })()


  return function processModuleFilter(module: any) {

    const modulePath = module['path']

    // 这个文件baseBundle已经包含了, 如果多次打包, 运行的时候会报错
    if (modulePath.endsWith('/node_modules/metro/src/lib/polyfills/require.js')) {
      return false
    }

    const moduleId = moduleIdFactory(modulePath)
    if (filterModules.has(moduleId)) {
      const version = modulesVersionHelper.getVersion(modulePath)

      if (filterModulesVersion.has(`${moduleId}@${version}`)) {
        throw new Error(`module ${modulePath} version dismatch: ${moduleId}@${version}`)
        process.exit(1)
      }

      return false
    }


    /*
     * FIXME 更新0.62.2的hook
     */
    if (modulePath.indexOf('__prelude__') >= 0) {
      // const hook = Buffer.from(process.env.HYEXT_RN_HOOK || '', 'base64')
      // const hookText = hook.toString('ascii')
      // module.output[0]['data']['code'] = hookText
      return true;
    }

    let code = module?.output?.[0]?.['data']?.['code'];
    if (!code) return true

    /*
     * FIXME 优化这段逻辑, 修复非压缩状态下替换失效的问题
     * 应该是放到 transform/babel 里比较合适
     */
    const nameMatch = code.match(/name[ ]*:[ ]*"(\S+?)"/);
    if (nameMatch && nameMatch.length >= 1) {
      const name = nameMatch[1];

      const typeMatch = code.match(/type[ ]*:[ ]*"(\S+?)"/);
      if (typeMatch && typeMatch.length >= 1) {
        const type = typeMatch[1];

        const relativeMatch = code.match(/httpServerLocation[ ]*:[ ]*"(\S+?)"/);
        if (relativeMatch && relativeMatch.length >= 1) {
          const relativePath = relativeMatch[1];

          //read image baseURI from process.env
          const uri = imageBaseURL + relativePath + '/' + name + '.' + type
          code = code.replace(/\.exports=(\S+)}/, `.exports={uri:"${uri}"}}`)
          module.output[0]['data']['code'] = code
        }
      }
    }

    return true;
  }
}

module.exports = filterModuleHOF
